/**
 * File: EmployeeDatabase.java
 * This file contains a sample program for how we could use the Employee
 * class.  It is an employee database program that allows the user
 * to add, look up, and update employees.
 */

import java.util.*;
import acm.program.*;

public class EmployeeDatabase extends ConsoleProgram {

	private static final int ADD_EMPLOYEE = 1;
	private static final int LOOKUP_EMPLOYEE = 2;
	private static final int PROMOTE_EMPLOYEE = 3;
	private static final int CHANGE_SALARY = 4;
	private static final int QUIT = 5;

	public void run() {
		HashMap<String, Employee> employees = 
				new HashMap<String, Employee>();
		println("Welcome to Employee Database(TM)!");

		while (true) {
			int choice = promptUser();
			if (choice == ADD_EMPLOYEE) {
				addEmployee(employees);
			} else if (choice == LOOKUP_EMPLOYEE) {
				lookupEmployee(employees);
			} else if (choice == PROMOTE_EMPLOYEE) {
				promoteEmployee(employees);
			} else if (choice == CHANGE_SALARY) {
				changeSalary(employees);
			} else if (choice == QUIT) {
				break;
			}
		}
	}

	/** Prompts the user for a choice of action, and reprompts them if
	 * the choice is invalid.  Returns the number the user selected.
	 */
	private int promptUser() {
		println("Please select from one of the following options:");
		println("\t" + ADD_EMPLOYEE + ": Add employee");
		println("\t" + LOOKUP_EMPLOYEE + ": Lookup employee");
		println("\t" + PROMOTE_EMPLOYEE + ": Promote employee");
		println("\t" + CHANGE_SALARY + ": Change employee salary");
		println("\t" + QUIT + ": Quit");

		int response = readInt("? ");
		while (response < ADD_EMPLOYEE && response > QUIT) {
			response = readInt("Invalid choice.\t?");
		}

		return response;
	}

	/*
	 * Prompts the user for an employee name, and reprompts them if the
	 * name is invalid.  Returns the employee with that name.
	 */
	private Employee getEmployee(HashMap<String, Employee> employees) {
		String name = readLine("Employee name: ");
		while (!employees.containsKey(name)) {
			println("Sorry, there's no employee with that name.");
			name = readLine("Employee name: ");
		}

		return employees.get(name);
	}

	/* Prompts the user for info and adds a new employee to our map. */
	private void addEmployee(HashMap<String, Employee> employees) {
		String name = readLine("Employee name? ");
		String title = readLine("Employee title? ");
		Employee newEmployee = new Employee(name, title);
		employees.put(name, newEmployee);		
	}

	/* Prints out information for an entered employee. */
	private void lookupEmployee(HashMap<String, Employee> employees) {
		Employee employee = getEmployee(employees);
		println("Employee: " + employee.getName() + " (" +
				employee.getTitle() + ")");
		println("Salary: $" + employee.getSalary());
	}

	/* Promotes the entered employee and prints out their new info. */
	private void promoteEmployee(HashMap<String, Employee> employees) {
		Employee employee = getEmployee(employees);
		employee.promote();
		println(employee.getName() + " promoted to " + 
				employee.getTitle() + "! (Salary $" +
				employee.getSalary() + ")");
	}

	/* Updates the salary for the entered employee. */
	private void changeSalary(HashMap<String, Employee> employees) {
		Employee employee = getEmployee(employees);
		int newSalary = readInt("New salary? ");
		employee.setSalary(newSalary);
	}
}

